@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with running Exim as a daemon */
@@ -618,8 +618,6 @@ setsid() can be #defined as a no-op, or as something else. */
 
 if (debug_level <= 0)
   {
-  pid_t pid;
-
   log_close_all();  /* Just in case anything was logged earlier */
   search_tidyup();  /* Just in case any were used in reading the config. */
   close(0);         /* Get rid of stdin/stdout/stderr */
@@ -627,17 +625,20 @@ if (debug_level <= 0)
   close(2);
   log_stderr = NULL;  /* So no attempt to copy paniclog output */
 
-  /* Fork, in case current process is a process group leader (see 'man
-  setsid' for an explanation). */
+  /* If the parent process of this one has pid == 1, we are re-initializing the
+  daemon as the result of a SIGHUP. In this case, there is no need to do any
+  forking, because the controlling terminal has long gone. Otherwise, fork,
+  in case current process is a process group leader (see 'man setsid' for an
+  explanation). */
 
-  pid = fork();
-  if (pid < 0) log_write(0, LOG_PANIC_DIE, "fork() failed: %s",
-    strerror(errno));
-  if (pid > 0) exit(EXIT_SUCCESS);      /* in parent process, just exit */
-
-  /* Release controlling terminal */
-
-  (void)setsid();
+  if (getppid() != 1)
+    {
+    pid_t pid = fork();
+    if (pid < 0) log_write(0, LOG_PANIC_DIE, "fork() failed: %s",
+      strerror(errno));
+    if (pid > 0) exit(EXIT_SUCCESS);      /* in parent process, just exit */
+    (void)setsid();                       /* release controlling terminal */
+    }
   }
 
 /* Ensure root privilege. It will only not exist at this stage if seteuid
