@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -752,6 +752,15 @@ if (*s == '=' || *s == '>' || *s == '<')
 
 else s = read_name(name, 256, s, "_");
 
+/* If we haven't read a name, it means some non-alpha character is first. */
+
+if (name[0] == 0)
+  {
+  expand_string_message = string_sprintf("condition name expected, "
+    "but found \"%.16s\"", s);
+  return NULL;
+  }
+
 /* def: tests for a non-zero or non-NULL variable, or for an existing
 header */
 
@@ -818,10 +827,10 @@ else if (strcmp(name, "exists") == 0
   char *sub;
   struct stat statbuf;
   while (isspace((uschar)*s)) s++;
-  if (*s != '{') goto COND_FAILED_CURLY;
+  if (*s != '{') goto COND_FAILED_CURLY_START;
   sub = expand_string_internal(s+1, TRUE, &s, FALSE);
   if (sub == NULL) return NULL;
-  if (*s++ != '}') goto COND_FAILED_CURLY;
+  if (*s++ != '}') goto COND_FAILED_CURLY_END;
   if (yield != NULL)
     {
     #ifdef SUPPORT_PAM
@@ -869,10 +878,16 @@ else if (strcmp(name, "eq") == 0 ||
   for (i = 0; i < 2; i++)
     {
     while (isspace((uschar)*s)) s++;
-    if (*s != '{') goto COND_FAILED_CURLY;
+    if (*s != '{')
+      {
+      if (i == 0) goto COND_FAILED_CURLY_START;
+      expand_string_message = string_sprintf("missing 2nd string in {} "
+        "after \"%s\"", name);
+      return NULL;
+      }
     sub[i] = expand_string_internal(s+1, TRUE, &s, yield == NULL);
     if (sub[i] == NULL) return NULL;
-    if (*s++ != '}') goto COND_FAILED_CURLY;
+    if (*s++ != '}') goto COND_FAILED_CURLY_END;
 
     /* Convert to numerical if required */
 
@@ -981,16 +996,27 @@ else if (strcmp(name, "or") == 0 || strcmp(name, "and") == 0)
   int comb = and;
 
   while (isspace((uschar)*s)) s++;
-  if (*s++ != '{') goto COND_FAILED_CURLY;
+  if (*s++ != '{') goto COND_FAILED_CURLY_START;
 
   for (;;)
     {
     while (isspace((uschar)*s)) s++;
-    if (*s != '{') break;
+    if (*s == '}') break;
+    if (*s != '{')
+      {
+      expand_string_message = string_sprintf("subcondition in {} expected "
+        "inside \"%s{...}\" condition", name);
+      return NULL;
+      }
     s = eval_condition(s+1, ptr);
     if (s == NULL) return NULL;
     while (isspace((uschar)*s)) s++;
-    if (*s++ != '}') goto COND_FAILED_CURLY;
+    if (*s++ != '}')
+      {
+      expand_string_message = string_sprintf("missing } at end of condition "
+        "inside \"%s\" group", name);
+      return NULL;
+      }
     if (yield != NULL)
       {
       if (and)
@@ -1006,9 +1032,8 @@ else if (strcmp(name, "or") == 0 || strcmp(name, "and") == 0)
       }
     }
 
-  if (*s++ != '}') goto COND_FAILED_CURLY;
   if (yield != NULL) *yield = (comb == testfor);
-  return s;
+  return ++s;
   }
 
 /* Unknown type of condition */
@@ -1016,10 +1041,15 @@ else if (strcmp(name, "or") == 0 || strcmp(name, "and") == 0)
 expand_string_message = string_sprintf("unknown condition \"%s\"", name);
 return NULL;
 
-/* Bracketing error */
+/* Missing braces at start and end of data */
 
-COND_FAILED_CURLY:
-expand_string_message = "missing or misplaced { or }";
+COND_FAILED_CURLY_START:
+expand_string_message = string_sprintf("missing { after \"%s\"", name);
+return NULL;
+
+COND_FAILED_CURLY_END:
+expand_string_message = string_sprintf("missing } at end of \"%s\" condition",
+  name);
 return NULL;
 }
 
@@ -1484,7 +1514,7 @@ while (*s != 0)
     {
     int n;
     s = read_number(&n, s);
-    if (n <= expand_nmax)
+    if (n >= 0 && n <= expand_nmax)
       yield = string_cat(yield, &size, &ptr, expand_nstring[n],
         expand_nlength[n]);
     continue;
@@ -1510,7 +1540,7 @@ while (*s != 0)
       expand_string_message = "} expected after number";
       goto EXPAND_FAILED;
       }
-    if (n <= expand_nmax)
+    if (n >= 0 && n <= expand_nmax)
       yield = string_cat(yield, &size, &ptr, expand_nstring[n],
         expand_nlength[n]);
     continue;
