@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -125,7 +125,7 @@ extern void  log_close_all(void);
 extern void  log_write(int, int, char *format, ...);
 
 extern BOOL  match_address_list(char *, int, char **, int, int *, BOOL);
-extern int   match_exists(char *);
+extern int   match_exists(char *, char **);
 extern BOOL  match_isinlist(char *, char **, BOOL, BOOL, char **);
 extern BOOL  match_check_string(char *, char *, int, BOOL, BOOL, char **);
 extern int   match_sender(char *);
@@ -159,6 +159,7 @@ extern char *parse_qp(char *);
 extern BOOL  queue_action(char *, int, char **, int, int);
 extern void  queue_check_only(void);
 extern void  queue_list(int, char **, int);
+extern void  queue_count(void);
 extern void  queue_run(char *, char *, BOOL);
 
 extern int   random_number(int);
