@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* All the global variables are defined together in this one module, so that
@@ -53,6 +53,8 @@ optionlist optionlist_auths[] = {
                  (void *)(offsetof(auth_instance, driver_name)) },
   { "public_name",   opt_stringptr | opt_public,
                  (void *)(offsetof(auth_instance, public_name)) },
+  { "server_mail_auth_condition", opt_stringptr | opt_public,
+                 (void *)(offsetof(auth_instance, mail_auth_condition)) },
   { "server_set_id", opt_stringptr | opt_public,
                  (void *)(offsetof(auth_instance, set_id)) }
 };
@@ -148,6 +150,7 @@ auth_instance auth_defaults   = {
    NULL,                      /* driver_name */
    NULL,                      /* public_name */
    NULL,                      /* set_id */
+   NULL,                      /* server_mail_auth_condition */
    FALSE,                     /* client */
    FALSE                      /* server */
 };
@@ -264,11 +267,6 @@ director_instance director_defaults = {
 };
 
 char  *dns_again_means_nonexist = NULL;
-#if HAVE_IPV6
-BOOL   dns_ipv4_lookup        = FALSE;
-#else
-BOOL   dns_ipv4_lookup        = TRUE;
-#endif
 int    dns_retrans            = 0;
 int    dns_retry              = 0;
 char  *domain_data            = NULL;
@@ -386,6 +384,11 @@ int    ignore_errmsg_errors_after = 0;
 BOOL   ignore_fromline_local  = FALSE;
 char  *ignore_fromline_hosts  = NULL;
 char  *interface_address      = NULL;
+#if HAVE_IPV6
+BOOL   ipv4_address_lookup    = FALSE;
+#else
+BOOL   ipv4_address_lookup    = TRUE;
+#endif
 
 int    journal_fd             = -1;
 
@@ -822,7 +825,7 @@ char  *warnmsg_recipients     = NULL;
 char  *verify_address_parse_error = NULL;
 char  *verify_forced_errmsg   = NULL;
 BOOL   verify_only            = FALSE;
-char  *version_copyright      = "Copyright (c) University of Cambridge 2000";
+char  *version_copyright      = "Copyright (c) University of Cambridge 2001";
 char  *version_date           = "?";
 char  *version_cnumber        = "????";
 char  *version_string         = "?";
