@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* All the global variables are defined together in this one header, so that
@@ -181,7 +181,6 @@ extern director_info directors_available[]; /* Vector of available directors */
 extern director_instance *directors;  /* Chain of instantiated directors */
 extern director_instance director_defaults; /* Default values */
 extern char  *dns_again_means_nonexist; /* Domains that are badly set up */
-extern BOOL   dns_ipv4_lookup;        /* If TRUE, don't look for AAAA & A6 */
 extern int    dns_retrans;            /* Retransmission time setting */
 extern int    dns_retry;              /* Number of retries */
 extern char  *domain_data;            /* From domain lookup */
@@ -261,6 +260,7 @@ extern int    ignore_errmsg_errors_after; /* Or keep them for this time. */
 extern BOOL   ignore_fromline_local;  /* Local SMTP ignore fromline */
 extern char  *ignore_fromline_hosts;  /* Hosts permitted to send "From " */
 extern char  *interface_address;      /* Interface on incoming call */
+extern BOOL   ipv4_address_lookup;    /* If TRUE, don't look for AAAA & A6 */
 
 extern int    journal_fd;             /* Fd for journal file */
 
