@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for finding hosts, either by gethostbyname, gethostbyaddr, or via
@@ -16,6 +16,38 @@ concerned with hosts and addresses. */
 #include "exim.h"
 
 
+#ifdef USE_INET_NTOA_FIX
+/*************************************************
+*         Replacement for broken inet_ntoa()     *
+*************************************************/
+
+/* On IRIX systems, gcc uses a different structure passing convention to the
+native libraries. This causes inet_ntoa() to always yield 0.0.0.0 or
+255.255.255.255. To get round this, we provide a private version of the
+function here. It is used only if USE_INET_NTOA_FIX is set, which should happen
+only when gcc is in use on an IRIX system. Code send to me by J.T. Breitner,
+with these comments:
+
+  code by Stuart Levy
+  as seen in comp.sys.sgi.admin
+
+Arguments:  sa  an in_addr structure
+Returns:        pointer to static text string
+*/
+
+char *
+inet_ntoa(struct in_addr sa)
+{
+static char addr[20];
+sprintf(addr, "%d.%d.%d.%d",
+        ((unsigned char *)&sa.s_addr)[0],
+        ((unsigned char *)&sa.s_addr)[1],
+        ((unsigned char *)&sa.s_addr)[2],
+        ((unsigned char *)&sa.s_addr)[3]);
+  return addr;
+}
+#endif
+
 
 
 /*************************************************
@@ -988,19 +1020,24 @@ int
 host_find_byname(host_item *host, char *ignore_target_hosts,
   char **fully_qualified_name, BOOL use_malloc)
 {
-int i, yield;
+int i, yield, times;
 char **addrlist;
 host_item *last = NULL;
 
-/* In an IPv6 world, we need to scan for both kinds of address, so go
-round the loop twice. Note that we have ensured that AF_INET6 is defined
-even in an IPv4 world, which makes for slightly tidier code. */
+/* In an IPv6 world, we need to scan for both kinds of address, so go round the
+loop twice. Note that we have ensured that AF_INET6 is defined even in an IPv4
+world, which makes for slightly tidier code. However, if ipv4_address_lookup is
+set (originally to disable IPv6 DNS lookups), we also just do IPv4 lookups
+here. */
 
 #if HAVE_IPV6
-int af = AF_INET6;
-int times = 2;
+int af;
+if (ipv4_address_lookup)
+  { af = AF_INET; times = 1; }
+else
+  { af = AF_INET6; times = 2; }
 #else
-int times = 1;
+times = 1;
 #endif
 
 /* Initialize the flag that gets set for DNS syntax check errors, so that the
@@ -1013,18 +1050,18 @@ host_find_failed_syntax = FALSE;
 for (i = 1; i <= times; i++)
   {
   BOOL ipv4_addr;
+  struct hostent *hostdata;
 
   #if HAVE_IPV6
     #if HAVE_GETIPNODEBYNAME
     int error_num;
-    struct hostent *hostdata =
-      getipnodebyname(host->name, af, 0, &error_num);
+    hostdata = getipnodebyname(host->name, af, 0, &error_num);
     #else
-    struct hostent *hostdata = gethostbyname2(host->name, af);
+    hostdata = gethostbyname2(host->name, af);
     #endif
-  af = AF_INET;
+  af = AF_INET;   /* For next time round the loop */
   #else
-  struct hostent *hostdata = gethostbyname(host->name);
+  hostdata = gethostbyname(host->name);
   #endif
 
   if (hostdata == NULL || (hostdata->h_addr_list)[0] == NULL) continue;
@@ -1049,8 +1086,8 @@ for (i = 1; i <= times; i++)
       host_ntoa(ipv4_addr? AF_INET:AF_INET6, *addrlist, NULL, NULL);
 
     if (ignore_target_hosts != NULL &&
-        verify_check_this_host(&ignore_target_hosts, FALSE, host->name, text_address,
-          NULL))
+        verify_check_this_host(&ignore_target_hosts, FALSE, host->name,
+          text_address, NULL))
       {
       DEBUG(8) debug_printf("ignored host %s [%s]\n", host->name, text_address);
       continue;
@@ -1179,11 +1216,11 @@ if (allow_ip && string_is_ip_address(host->name, NULL) != 0)
   }
 
 /* On an IPv6 system, go round the loop three times, looking for A6 and AAAA
-records the first two times. On an IPv4 system, go round the loop once only,
-looking only for A records. */
+records the first two times, unless ipv4_address_lookup is set. On an IPv4
+system, go round the loop once only, looking only for A records. */
 
 #if HAVE_IPV6
-i = dns_ipv4_lookup? 0 : 2;
+i = ipv4_address_lookup? 0 : 2;
 #else
 i = 0;
 #endif
