@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for matching strings */
@@ -382,6 +382,7 @@ Argument:
            or NULL; a leading "!" causes the test for that file to be for
            non-existence. A leading "+" causes an EACCES from the stat()
            call to be treated as if it were ENOENT.
+  error    a pointer to an anchor for an error text in the case of a DEFER
 
 Returns:   OK if s == NULL or all files' existence is as required;
            DEFER if the existence of at least one of the files is
@@ -392,7 +393,7 @@ Returns:   OK if s == NULL or all files' existence is as required;
 */
 
 int
-match_exists(char *s)
+match_exists(char *s, char **error)
 {
 char *ss, *file, *listptr;
 char buffer[1024];
@@ -408,9 +409,8 @@ if (s == NULL) return OK;
 ss = expand_string(s);
 if (ss == NULL)
   {
-  if (search_find_defer) return DEFER;
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "expansion of %s failed: %s", s,
-    expand_string_message);
+  *error = string_sprintf("expansion failure: %s", expand_string_message);
+  return DEFER;
   }
 
 DEBUG(9) debug_printf("require_files = %s\n", ss);
@@ -462,8 +462,11 @@ while ((file = string_nextinlist(&listptr, &sep, buffer, sizeof(buffer)))
     a non-absolute path. */
 
     if (strchr(file, '/') != NULL)
-      log_write(0, LOG_MAIN|LOG_PANIC_DIE, "file name for existence test is "
+      {
+      *error = string_sprintf("file name for existence test is "
         "not fully qualified: %s", file);
+      return DEFER;
+      }
 
     /* If this is the first change of uid/gid, remember what we have to
     go back to. */
@@ -531,6 +534,7 @@ while ((file = string_nextinlist(&listptr, &sep, buffer, sizeof(buffer)))
   if (rc < 0 && save_errno != ENOENT && save_errno != ENOTDIR &&
       (!eperm_is_absence || save_errno != EACCES))
     {
+    *error = string_sprintf("%s", strerror(save_errno));
     yield = DEFER;
     break;
     }
