@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions that operate on the input queue. */
@@ -584,6 +584,35 @@ if (!recurse)
 
 
 
+
+/************************************************
+*         Count messages on the queue           *
+************************************************/
+
+/* Called as a result of -bpc
+
+Arguments:  none
+Returns:    nothing
+*/
+
+void
+queue_count(void)
+{
+int subcount;
+int count = 0;
+queue_filename *f = NULL;
+uschar subdirs[64];
+f = queue_get_spool_list(
+        -1,             /* entire queue */
+        subdirs,        /* for holding sub list */
+        &subcount,      /* for subcount */
+        FALSE);         /* not random */
+for (; f != NULL; f = f->next) count++;
+fprintf(stdout, "%d\n", count);
+}
+
+
+
 /************************************************
 *          List extra deliveries                *
 ************************************************/
@@ -1119,7 +1148,7 @@ switch(action)
         else if (action == MSG_MARK_DELIVERED)
           {
           for (i = 0; i < recipients_count; i++)
-            if (strcmp(recipients_list[i].address, receiver) == 0) break;
+            if (strcmpic(recipients_list[i].address, receiver) == 0) break;
           if (i >= recipients_count)
             {
             printf("- error while %s:\n  %s is not a recipient:"
@@ -1128,7 +1157,7 @@ switch(action)
             }
           else
             {
-            tree_add_nonrecipient(receiver, FALSE);
+            tree_add_nonrecipient(recipients_list[i].address, FALSE);
             log_write(0, LOG_MAIN, "address <%s> marked delivered by %s",
               receiver, username);
             }
