@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with routing, and the list of generic router options. */
@@ -555,6 +555,7 @@ addr->route_domain = addr->domain;
 for (r = routers; more && r != NULL; r = r->next)
   {
   char *old_route_domain = addr->route_domain;
+  char *error;
   address_item *parent;
   BOOL loop_detected = FALSE;
   int yield = FALSE;
@@ -606,11 +607,11 @@ for (r = routers; more && r != NULL; r = r->next)
   as the string is expanded - and if they succeed, do any necessary sender
   checks, and run the router. Otherwise set an appropriate yield value. */
 
-  switch(match_exists(r->require_files))
+  switch(match_exists(r->require_files, &error))
     {
     case DEFER:
     addr->message = string_sprintf("file existence defer in %s router: %s",
-      r->name, strerror(errno));
+      r->name, error);
     yield = DEFER;
     break;
 
