@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* General functions concerned with transportation, and generic options for all
@@ -528,17 +528,52 @@ if ((options & topt_no_headers) == 0)
 
     for (p = addr; p != NULL; p = p->next)
       {
+      struct aci {
+        struct aci *next;
+        address_item *ptr;
+        };
       address_item *pp, *dup;
+      struct aci *plist = NULL;
+      void *reset_point = store_get(0);
 
-      /* Scan for any duplicates that relate to this address. */
+      for (pp = p; pp->parent != NULL; pp = pp->parent);
+
+      /* Scan for any duplicates that relate to this address, and include their
+      ultimate parents. However, we want to avoid duplication in the list,
+      which can arise for example when A->B,C and then both B and C alias to D.
+      This can also happen when there are unseen drivers in use. Keep a list of
+      addresses that have been output. This is messy, but in most cases the
+      list will be very short. */
 
       for (dup = addr_duplicate; dup != NULL; dup = dup->next)
         {
-        if (strcmp(dup->unique, p->unique) == 0)
+        if (strcmpic(dup->orig, p->orig) == 0)
           {
           address_item *dup_orig;
+          struct aci *ppp;
+
           for (dup_orig = dup; dup_orig->parent != NULL;
             dup_orig = dup_orig->parent);
+
+          /* If the parent is the parent of the main address item, skip it.
+          Otherwise, scan the list of already output parents, and skip if
+          we've done this one already. */
+
+          if (dup_orig == pp) continue;
+          for (ppp = plist; ppp != NULL; ppp = ppp->next)
+            {
+            if (dup_orig == ppp->ptr) break;
+            }
+          if (ppp != NULL) continue;
+
+          /* Add this duplicate's parent to the done list, and then output
+          its address. */
+
+          ppp = store_get(sizeof(struct aci));
+          ppp->next = plist;
+          plist = ppp;
+          ppp->ptr = dup_orig;
+
           sprintf(buffer + len, "%.200s,\n ", dup_orig->orig);
           len += (int)strlen(dup_orig->orig) + 3;
           if (!write_chunk(fd, buffer, len, use_crlf)) return FALSE;
@@ -547,9 +582,10 @@ if ((options & topt_no_headers) == 0)
           }
         }
 
-      /* Now list the ultimate parent of this address */
+      store_reset(reset_point);
+
+      /* Now list the ultimate parent of this main address */
 
-      for (pp = p; pp->parent != NULL; pp = pp->parent);
       strncpy(buffer + len, pp->orig, 200);
       len += (int)strlen(pp->orig);
 
