@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -277,10 +277,17 @@ for (i = 0; i < 2; i++)
       {
       if (ob->quota_value <= 0)
         {
+        /* We used to object to a percentage setting with no quota set,
+        but there was a request just to ignore, so that is what is now done. */
+
+        /****
         *errmsg = string_sprintf("quota_warn_threshold cannot be specified as "
           "a percentage (%s) unless quota is also set for %s transport",
             s, tblock->name);
         return FAIL;
+        ****/
+
+        d = 0;
         }
       if ((int)d < 0 || (int)d > 100)
         {
@@ -431,7 +438,9 @@ if (ob->create_file_string != NULL)
   ob->create_file = value;
   }
 
-/* If quota_warn_threshold is set, set up default for warn_message */
+/* If quota_warn_threshold is set, set up default for warn_message. It may
+not be used if the actual threshold for a given delivery ends up as zero,
+of if it's given as a percentage and there's no quota setting. */
 
 if (ob->quota_warn_threshold != NULL)
   {
@@ -488,7 +497,8 @@ BSD thing that uses a TCP/IP protocol for communication. A message consisting
 of the text "user@offset" must be sent, where offset is the place in the
 mailbox where new mail starts. There is no scope for telling it which file to
 look at, which makes it a less than useful if mail is being delivered into a
-non-standard place such as the user's home directory.
+non-standard place such as the user's home directory. In fact, it doesn't seem
+to pay much attention to the offset.
 
 Arguments:
   user       user name
@@ -500,25 +510,14 @@ Returns:     nothing
 static void
 notify_comsat(char *user, int offset)
 {
-int fd;
-struct sockaddr_in sa;
-struct hostent *hp;
 struct servent *sp;
+host_item host;
+host_item *h;
 char buffer[256];
 
 DEBUG(2) debug_printf("notify_comsat called\n");
 
-if ((hp = gethostbyname("localhost")) == NULL)
-  {
-  DEBUG(2) debug_printf("\"localhost\" unknown\n");
-  return;
-  }
-
-if (hp->h_addrtype != AF_INET)
-  {
-  DEBUG(2) debug_printf("local host not TCP/IP\n");
-  return;
-  }
+sprintf(buffer, "%.200s@%d\n", user, offset);
 
 if ((sp = getservbyname("biff", "udp")) == NULL)
   {
@@ -526,25 +525,93 @@ if ((sp = getservbyname("biff", "udp")) == NULL)
   return;
   }
 
-sa.sin_port = sp->s_port;
-sa.sin_family = hp->h_addrtype;
-memcpy(&sa.sin_addr, hp->h_addr, hp->h_length);
+host.name = "localhost";
+host.next = NULL;
 
-if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
+
+/* This code is all set up to look up "localhost" and use all its addresses
+until one succeeds. However, it appears that at least on some systems, comsat
+doesn't listen on the ::1 address. So for the moment, just force the address to
+be 127.0.0.1. At some future stage, when IPv6 really is superseding IPv4, this
+can be changed. */
+
+/******
+if (host_find_byname(&host, NULL, NULL, FALSE) == HOST_FIND_FAILED)
   {
-  DEBUG(2) debug_printf("failed to create comsat socket: %s\n",
-    strerror(errno));
+  DEBUG(2) debug_printf("\"localhost\" unknown\n");
   return;
   }
+******/
 
-sprintf(buffer, "%.200s@%d\n", user, offset);
-if (sendto(fd, buffer, (int)strlen(buffer) + 1, 0, (struct sockaddr *)(&sa),
-    sizeof(sa)) < 0)
+host.address = "127.0.0.1";
+
+
+for (h = &host; h != NULL; h = h->next)
   {
-  DEBUG(2) debug_printf("send to comsat failed: %s\n", strerror(errno));
-  }
+  struct sockaddr *s_ptr;
+  struct sockaddr_in s_in4;
+  int host_af = (strchr(h->address, ':') != NULL)? AF_INET6 : AF_INET;
+  int sock, s_len, rc;
 
-close(fd);
+  #if HAVE_IPV6
+  struct sockaddr_in6 s_in6;
+
+  /* For an IPv6 address, use an IPv6 sockaddr structure. */
+
+  if (host_af == AF_INET6)
+    {
+    s_ptr = (struct sockaddr *)&s_in6;
+    s_len = sizeof(s_in6);
+    }
+  else
+  #endif
+
+  /* For an IPv4 address, use an IPv4 sockaddr structure,
+  even on an IPv6 system. */
+
+    {
+    s_ptr = (struct sockaddr *)&s_in4;
+    s_len = sizeof(s_in4);
+    }
+
+  DEBUG(2) debug_printf("calling comsat on %s\n", h->address);
+
+  sock = socket(host_af, SOCK_DGRAM, 0);
+  if (sock < 0)
+    {
+    DEBUG(2) debug_printf("failed to create comsat socket: %s\n",
+      strerror(errno));
+    continue;
+    }
+
+  #if HAVE_IPV6
+  if (host_af == AF_INET6)
+    {
+    memset(&s_in6, 0, sizeof(s_in6));
+    s_in6.sin6_family = AF_INET6;
+    s_in6.sin6_port = sp->s_port;
+    if (inet_pton(host_af, h->address, &s_in6.sin6_addr) != 1)
+      log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", h->address);
+    }
+  else
+  #endif
+
+  /* Set up a remote IPv4 address */
+
+    {
+    memset(&s_in4, 0, sizeof(s_in4));
+    s_in4.sin_family = AF_INET;
+    s_in4.sin_port = sp->s_port;
+    s_in4.sin_addr.s_addr = (S_ADDR_TYPE)inet_addr(h->address);
+    }
+
+  rc = sendto(sock, buffer, (int)strlen(buffer) + 1, 0, s_ptr, s_len);
+  close(sock);
+
+  if (rc >= 0) break;
+  DEBUG(2) debug_printf("send to comsat failed for %s: %s\n", strerror(errno),
+    h->address);
+  }
 }
 
 
