@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with verifying things. */
@@ -321,6 +321,10 @@ while (addr_new != NULL)
         uschar inbuffer[1024];
         uschar outbuffer[1024];
 
+        /* Skip this host if we don't have an IP address for it. */
+
+        if (host->address == NULL) continue;
+
         /* Set up the buffer for reading SMTP response packets. */
 
         inblock.buffer = inbuffer;
@@ -514,8 +518,11 @@ while (addr_new != NULL)
     one_continue set TRUE (aliasfile and smartuser do, forwardfile does not). */
 
     if (!full_info &&
-         (addr_new == NULL || addr_new->next != NULL ||
-          addr->director == NULL || !addr->director->one_continue))
+         (addr_new == NULL ||                    /* No new address */
+          addr_new->next != NULL ||              /* More than one new address */
+          testflag(addr_new, af_pfr) ||          /* New address is pfr */
+          addr->director == NULL ||              /* Address was routed */
+          !addr->director->one_continue))        /* Director said stop */
       {
       if (f != NULL) fprintf(f, "%s %s\n",
         addr->orig,
@@ -528,11 +535,7 @@ while (addr_new != NULL)
 /* Display the full results of the successful directing and routing, including
 any generated addresses. Control gets here only when full_info is set, which
 requires f not to be NULL, and this occurs only when a top-level verify is
-called with the debugging switch on, or as a result of an SMTP EXPN command
-(permitted only for specified hosts/nets). For verification, if the debugging
-value is greater than 1, show everything; for 1 (which is also -v) show what
-"the normal user" might be interested in. For address testing, always show
-everything.
+called with the debugging switch on.
 
 If there are no local and no remote addresses, and there were no pipes, files,
 or autoreplies, and there were no errors or deferments, the message is to be
@@ -541,8 +544,6 @@ discarded, usually because of the use of :blackhole: in an alias file. */
 if (allok && addr_local == NULL && addr_remote == NULL)
   fprintf(f, "mail to %s is discarded\n", s);
 
-full_info = debug_level > 1 || address_test_mode;
-
 for (addr_list = addr_local, i = 0; i < 2; addr_list = addr_remote, i++)
   {
   while (addr_list != NULL)
@@ -567,27 +568,22 @@ for (addr_list = addr_local, i = 0; i < 2; addr_list = addr_remote, i++)
 
       if (strcmp(addr->domain, addr->route_domain) != 0)
         fprintf(f, " via domain %s", addr->route_domain);
-      fprintf(f, "\n");
+      fprintf(f, "\n  ");
 
-      if (full_info)
-        {
-        fprintf(f, "  ");
-        if (addr->director != NULL)
-          fprintf(f, "director = %s, ", addr->director->name);
-        fprintf(f, "router = %s, ", addr->router->name);
-        }
+      if (addr->director != NULL)
+        fprintf(f, "director = %s, ", addr->director->name);
+      fprintf(f, "router = %s, ", addr->router->name);
       }
 
     else if (addr->director != NULL)
       {
       fprintf(f, "\n  deliver to %s in domain %s\n",
         addr->local_part, addr->domain);
-      if (full_info) fprintf(f, "  director = %s, ", addr->director->name);
+      fprintf(f, "  director = %s, ", addr->director->name);
       }
 
-    if (full_info)
-      fprintf(f, "transport = %s\n", (addr->transport == NULL)? "unset" :
-        addr->transport->name);
+    fprintf(f, "transport = %s\n", (addr->transport == NULL)? "unset" :
+      addr->transport->name);
 
     /* Show any hosts that are set up by a router unless the transport
     is going to override them; fiddle a bit to get a nice format. */
@@ -615,11 +611,12 @@ for (addr_list = addr_local, i = 0; i < 2; addr_list = addr_remote, i++)
           fprintf(f, "[%s] ", h->address);
           len = (int)strlen(h->address);
           }
-        else
+        else if (!addr->transport->info->local)  /* Omit [unknown] for local */
           {
           fprintf(f, "[unknown] ");
           len = 7;
           }
+        else len = -3;
         while (len++ < maxaddlen) fprintf(f," ");
         if (h->mx >= 0) fprintf(f, "MX=%d", h->mx);
         if (h->status == hstatus_unusable) fprintf(f, " ** unusable **");
